@@ -1,3 +1,5 @@
+import configparser
+import glob
 import os
 import re
 from subprocess import Popen, PIPE
@@ -6,13 +8,14 @@ import requests
 import tarfile
 import zipfile
 
-from selenium import webdriver
+from appium.options.android import UiAutomator2Options
+from appium.options.ios import XCUITestOptions
 from selenium.webdriver.safari.options import Options
 from selenium.webdriver.remote.webdriver import WebDriver
 
 from pwdriver.val import CONFIG_NAME, CHROME, CHROMEDRIVER, CHROMEDRIVER_API, CHROMEDRIVER_NAME, DRIVER, \
     EDGE, EDGEDRIVER, EDGEDRIVER_API, EDGEDRIVER_NAME, GECKO, GECKODRIVER, GECKODRIVER_API, GECKODRIVER_NAME, \
-    OS_BIT, OS_NAME, ROOT_DIR, SAFARI, TAR_GZ, ZIP
+    OS_BIT, OS_NAME, ROOT_DIR, SAFARI, TAR_GZ, ZIP, ANDROID, IOS
 from pwdriver import util
 
 logger = util.get_logger('core')
@@ -20,7 +23,7 @@ driver_path = os.path.join(ROOT_DIR, DRIVER)
 dir_regex = r'\S+[dD]river'
 
 
-def _get_local_chrome_version() -> str:
+def __get_local_chrome_version() -> str:
     if OS_NAME == 'win':
         path = ['Program Files', 'Program Files (x86)']
         for loc in path:
@@ -47,19 +50,19 @@ def _get_local_chrome_version() -> str:
     return version
 
 
-def _get_latest_chrome_version(version) -> str:
+def __get_latest_chrome_version(version: str) -> str:
     latest_release = requests.get(f'{CHROMEDRIVER_API}/LATEST_RELEASE_{re.split(r"[.]", version)[0]}')
     global driver_path
     if not driver_path.endswith(latest_release.text):
         driver_path = os.path.join(driver_path, CHROME, latest_release.text)
-    logger.info(f'Latest Chromedriver version: {latest_release.text}')
+    logger.info(f'Latest {CHROMEDRIVER} version: {latest_release.text}')
     return latest_release.text
 
 
-def _download_chromedriver(version) -> None:
+def __download_chromedriver(version: str) -> None:
     global driver_path
     if not os.path.isfile(os.path.join(driver_path, CHROMEDRIVER_NAME)):
-        logger.info('Not found executable chromedriver. Chromedriver will be downloaded.')
+        logger.info(f'Not found executable {CHROMEDRIVER}. {CHROMEDRIVER} will be downloaded in \'{driver_path}\'.')
         download_url = f'{CHROMEDRIVER_API}/{version}/chromedriver_{OS_NAME}' + \
                        (OS_BIT if OS_NAME != 'win' else '32') + ZIP
         file = requests.get(download_url, stream=True)
@@ -71,21 +74,21 @@ def _download_chromedriver(version) -> None:
         zipfile.ZipFile(file_name).extractall(driver_path)
         os.remove(file_name)
         if not os.path.isfile(os.path.join(driver_path, CHROMEDRIVER_NAME)):
-            os.rename(os.path.join(driver_path,
-                                   util.get_pattern_matched_file(driver_path, dir_regex)[0]),
+            os.rename(os.path.join(driver_path, util.get_pattern_matched_file(driver_path, dir_regex)[0]),
                       os.path.join(driver_path, CHROMEDRIVER_NAME))
     else:
         logger.info(f'Executable driver found: ({os.path.abspath(driver_path)})')
 
 
 def setup_chromedriver() -> None:
-    _download_chromedriver(_get_latest_chrome_version(_get_local_chrome_version()))
+    __download_chromedriver(__get_latest_chrome_version(__get_local_chrome_version()))
     global driver_path
     util.set_file_executable(os.path.join(driver_path, CHROMEDRIVER_NAME))
     os.environ['PATH'] += f'{os.pathsep}{os.path.abspath(driver_path)}'
+    driver_path = os.path.join(ROOT_DIR, DRIVER)
 
 
-def _get_latest_gecko_version() -> str:
+def __get_latest_gecko_version() -> str:
     version = re.split(r'/+', requests.get(f'{GECKODRIVER_API}/latest', allow_redirects=True).url)[-1]
     global driver_path
     if not driver_path.endswith(version):
@@ -93,10 +96,10 @@ def _get_latest_gecko_version() -> str:
     return version
 
 
-def _download_geckodriver(version) -> None:
+def __download_geckodriver(version: str) -> None:
     global driver_path
     if not os.path.isfile(os.path.join(driver_path, GECKODRIVER_NAME)):
-        logger.info(f'Not found executable geckodriver. Geckodriver will be downloaded.')
+        logger.info(f'Not found executable {GECKODRIVER}. {GECKODRIVER} will be downloaded in \'{driver_path}\'.')
         postfix = OS_NAME + (OS_BIT if OS_NAME != 'mac' else 'os') + (ZIP if OS_NAME == 'win' else TAR_GZ)
         download_url = f'{GECKODRIVER_API}/download/{version}/geckodriver-{version}-{postfix}'
         file = requests.get(download_url, stream=True)
@@ -112,8 +115,7 @@ def _download_geckodriver(version) -> None:
             tar.extractall(driver_path)
             tar.close()
         if not os.path.isfile(os.path.join(driver_path, GECKODRIVER_NAME)):
-            os.rename(os.path.join(driver_path,
-                                   util.get_pattern_matched_file(driver_path, dir_regex)[0]),
+            os.rename(os.path.join(driver_path, util.get_pattern_matched_file(driver_path, dir_regex)[0]),
                       os.path.join(driver_path, GECKODRIVER_NAME))
         os.remove(file_name)
     else:
@@ -121,13 +123,14 @@ def _download_geckodriver(version) -> None:
 
 
 def setup_geckodriver() -> None:
-    _download_geckodriver(_get_latest_gecko_version())
+    __download_geckodriver(__get_latest_gecko_version())
     global driver_path
     util.set_file_executable(os.path.join(driver_path, GECKODRIVER_NAME))
     os.environ['PATH'] += f'{os.pathsep}{os.path.abspath(driver_path)}'
+    driver_path = os.path.join(ROOT_DIR, DRIVER)
 
 
-def _get_local_edge_version() -> str:
+def __get_local_edge_version() -> str:
     if OS_NAME == 'win':
         path = ['Program Files', 'Program Files (x86)']
         for loc in path:
@@ -154,12 +157,12 @@ def _get_local_edge_version() -> str:
     return version
 
 
-def _download_edgedriver(version) -> None:
+def __download_edgedriver(version: str) -> None:
     global driver_path
     if not driver_path.endswith(version):
         driver_path = os.path.join(driver_path, EDGE, version)
     if not os.path.isfile(os.path.join(driver_path, EDGEDRIVER_NAME)):
-        logger.info(f'Not found executable edgedriver. Edgedriver will be downloaded.')
+        logger.info(f'Not found executable {EDGEDRIVER}. {EDGEDRIVER} will be downloaded in \'{driver_path}\'.')
         download_url = f'{EDGEDRIVER_API}/{version}/edgedriver_{OS_NAME}{OS_BIT}{ZIP}'
         file = requests.get(download_url, stream=True)
         file_name = f'{EDGEDRIVER}{ZIP}'
@@ -169,8 +172,7 @@ def _download_edgedriver(version) -> None:
                 fd.write(chunk)
         zipfile.ZipFile(file_name).extractall(driver_path)
         if not os.path.isfile(os.path.join(driver_path, EDGEDRIVER_NAME)):
-            os.rename(os.path.join(driver_path,
-                                   util.get_pattern_matched_file(driver_path, dir_regex)[0]),
+            os.rename(os.path.join(driver_path, util.get_pattern_matched_file(driver_path, dir_regex)[0]),
                       os.path.join(driver_path, EDGEDRIVER_NAME))
         os.remove(file_name)
     else:
@@ -178,46 +180,65 @@ def _download_edgedriver(version) -> None:
 
 
 def setup_edgedriver() -> None:
-    _download_edgedriver(_get_local_edge_version())
+    __download_edgedriver(__get_local_edge_version())
     global driver_path
     util.set_file_executable(os.path.join(driver_path, EDGEDRIVER_NAME))
     os.environ['PATH'] += f'{os.pathsep}{os.path.abspath(driver_path)}'
+    driver_path = os.path.join(ROOT_DIR, DRIVER)
 
 
 class WebDriverFactory:
+    _automation_target = None
+    _mobile_caps = {}
+    _automation_local = None
+    _automation_url = None
+
     def __init__(self):
-        import configparser as cp
-        config = cp.ConfigParser()
-        import glob
+        config = configparser.ConfigParser()
+        config.optionxform = str
         config_path = glob.glob(os.path.join(ROOT_DIR, '**', CONFIG_NAME), recursive=True)
         if not config_path:
-            raise NotImplementedError(f'Not found \'{CONFIG_NAME}\' configuration file in directory.')
+            raise NotImplementedError(f'Not found \'{CONFIG_NAME}\' configuration file in \'{ROOT_DIR}\'.')
         config.read(config_path[0])
-        self._automation_browser = config.get('automation', 'automation.browser')
-        if self._automation_browser not in [CHROME, GECKO, EDGE, SAFARI]:
-            raise NotImplementedError(f'Unsupported browser name: {self._automation_browser}')
-        self._automation_local = util.parse_boolean(config.get('automation', 'automation.local'))
-        if not self._automation_local:
-            self._automation_url = config.get('automation', 'automation.url')
+        WebDriverFactory._automation_target = config.get('automation', 'target')
+        if WebDriverFactory._automation_target not in [CHROME, GECKO, EDGE, SAFARI, ANDROID, IOS]:
+            raise NotImplementedError(f'Unsupported browser name: {WebDriverFactory._automation_target}')
+        if WebDriverFactory._automation_target in [ANDROID, IOS]:
+            try:
+                WebDriverFactory._mobile_caps = dict(config.items('mobile'))
+            except configparser.NoSectionError:
+                raise NotImplementedError(f'Capabilities for {WebDriverFactory._automation_target} must exist.')
+        WebDriverFactory._automation_local = util.parse_boolean(config.get('automation', 'local'))
+        if not WebDriverFactory._automation_local:
+            WebDriverFactory._automation_url = config.get('automation', 'url')
 
-    def launch(self, options=None) -> WebDriver:
+    @classmethod
+    def launch(cls, options=None) -> WebDriver:
+        c = cls()
+        from selenium import webdriver
         options_dict = {
             CHROME: webdriver.ChromeOptions(),
             GECKO: webdriver.FirefoxOptions(),
             EDGE: webdriver.EdgeOptions(),
-            SAFARI: webdriver.safari.options.Options()
+            SAFARI: webdriver.safari.options.Options(),
+            ANDROID: UiAutomator2Options().load_capabilities(c._mobile_caps),
+            IOS: XCUITestOptions().load_capabilities(c._mobile_caps)
         }
-        if self._automation_local:
-            if self._automation_browser == CHROME:
+        if c._automation_local:
+            if c._automation_target == CHROME:
                 setup_chromedriver()
                 return webdriver.Chrome(options=options)
-            if self._automation_browser == GECKO:
+            if c._automation_target == GECKO:
                 setup_geckodriver()
                 return webdriver.Firefox(options=options)
-            if self._automation_browser == EDGE:
+            if c._automation_target == EDGE:
                 setup_edgedriver()
                 return webdriver.Edge(options=options)
-            if self._automation_browser == SAFARI:
-                return webdriver.Safari(options=options)
-        return webdriver.Remote(command_executor=self._automation_url,
-                                options=options if options is not None else options_dict[self._automation_browser])
+            if c._automation_target == SAFARI:
+                return webdriver.Safari(options=options if options is not None else options_dict[c._automation_target])
+        if c._automation_target not in [ANDROID, IOS]:
+            return webdriver.Remote(command_executor=c._automation_url,
+                                    options=options if options is not None else options_dict[c._automation_target])
+        from appium import webdriver
+        return webdriver.Remote(command_executor=c._automation_url,
+                                options=options_dict[c._automation_target])
