@@ -1,12 +1,12 @@
 import platform
 import sys
 
-platform_dict = {
+PLATFORM_DICT = {
     'Windows': 'win',
     'Darwin': 'mac',
     'Linux': 'linux'
 }
-OS_NAME = platform_dict[platform.system()]
+OS_NAME = PLATFORM_DICT[platform.system()]
 OS_BIT = '64' if platform.architecture()[0] == '64bit' else '32'
 EXE = '.exe'
 INI = '.ini'
@@ -28,6 +28,8 @@ GECKODRIVER = f'{GECKO}{DRIVER}'
 GECKODRIVER_NAME = f'{GECKODRIVER}{EXE}' if OS_NAME == 'win' else GECKODRIVER
 EDGEDRIVER_API = 'https://msedgedriver.azureedge.net'
 EDGE = 'edge'
-EDGEDRIVER = f'msedge{DRIVER}'
+EDGEDRIVER = f'ms{EDGE}{DRIVER}'
 EDGEDRIVER_NAME = f'{EDGEDRIVER}{EXE}' if OS_NAME == 'win' else EDGEDRIVER
 SAFARI = 'safari'
+ANDROID = 'android'
+IOS = 'ios'
